"""通用标注, 无法用于创建 MS对象"""

import re
import abc
import json
import contextlib
from io import BytesIO
from pathlib import Path
from base64 import b64decode
from datetime import datetime
from urllib.parse import urlparse
from dataclasses import field, asdict, dataclass
from typing import (
    TYPE_CHECKING,
    Any,
    Dict,
    List,
    Type,
    Union,
    Generic,
    Literal,
    TypeVar,
    Callable,
    Iterable,
    Optional,
    overload,
)

from nepattern import MatchMode, BasePattern, create_local_patterns
from nonebot.internal.adapter import Bot, Event, Message, MessageSegment

from .utils import fleep

if TYPE_CHECKING:
    from .message import UniMessage


TS = TypeVar("TS", bound="Segment")
TS1 = TypeVar("TS1", bound="Segment")


class UniPattern(BasePattern[TS, MessageSegment], Generic[TS]):
    additional: Optional[Callable[..., bool]] = None

    def __init__(self):
        origin: Type[TS] = self.__class__.__orig_bases__[0].__args__[0]  # type: ignore

        def _converter(_, seg: MessageSegment) -> Optional[TS]:
            if (res := self.solve(seg)) and not hasattr(res, "origin"):
                res.origin = seg
            return res

        super().__init__(
            mode=MatchMode.TYPE_CONVERT,
            origin=origin,
            converter=_converter,
            alias=origin.__name__,
            accepts=MessageSegment,
            validators=[self.additional] if self.additional else [],
        )

    def solve(self, seg: MessageSegment) -> Optional[TS]:
        raise NotImplementedError


class Segment:
    """基类标注"""

    if TYPE_CHECKING:
        origin: MessageSegment  # = field(init=False, repr=False, compare=False)

    def __str__(self):
        return f"[{self.__class__.__name__.lower()}]"

    def __repr__(self):
        attrs = ", ".join(f"{k}={v!r}" for k, v in self.data.items())
        return f"{self.__class__.__name__}({attrs})"

    @overload
    def __add__(self: TS, item: str) -> "UniMessage[Union[TS, Text]]": ...

    @overload
    def __add__(self: TS, item: Union[TS, Iterable[TS]]) -> "UniMessage[TS]": ...

    @overload
    def __add__(self: TS, item: Union[TS1, Iterable[TS1]]) -> "UniMessage[Union[TS, TS1]]": ...

    def __add__(self: TS, item: Union[str, Union[TS, TS1], Iterable[Union[TS, TS1]]]) -> "UniMessage":
        from .message import UniMessage

        return UniMessage(self) + item

    @overload
    def __radd__(self: TS, item: str) -> "UniMessage[Union[Text, TS]]": ...

    @overload
    def __radd__(self: TS, item: Union[TS, Iterable[TS]]) -> "UniMessage[TS]": ...

    @overload
    def __radd__(self: TS, item: Union[TS1, Iterable[TS1]]) -> "UniMessage[Union[TS1, TS]]": ...

    def __radd__(self: TS, item: Union[str, Union[TS, TS1], Iterable[Union[TS, TS1]]]) -> "UniMessage":
        from .message import UniMessage

        return UniMessage(item) + self

    def is_text(self) -> bool:
        return False

    @property
    def type(self) -> str:
        return self.__class__.__name__.lower()

    @property
    def data(self) -> Dict[str, Any]:
        try:
            return asdict(self)  # type: ignore
        except TypeError:
            return vars(self)


@dataclass
class Text(Segment):
    """Text对象, 表示一类文本元素"""

    text: str
    style: Optional[str] = field(default=None)

    def __post_init__(self):
        self.text = str(self.text)

    def is_text(self) -> bool:
        return True

    def __str__(self):
        return self.text


@dataclass
class At(Segment):
    """At对象, 表示一类提醒某用户的元素"""

    flag: Literal["user", "role", "channel"]
    target: str
    display: Optional[str] = field(default=None)


@dataclass
class AtAll(Segment):
    """AtAll对象, 表示一类提醒所有人的元素"""

    here: bool = field(default=False)


@dataclass
class Emoji(Segment):
    """Emoji对象, 表示一类表情元素"""

    id: str
    name: Optional[str] = field(default=None)


@dataclass
class Media(Segment):
    id: Optional[str] = field(default=None)
    url: Optional[str] = field(default=None)
    path: Optional[Union[str, Path]] = field(default=None)
    raw: Optional[Union[bytes, BytesIO]] = field(default=None)
    mimetype: Optional[str] = field(default=None)
    name: Optional[str] = field(default=None)

    def __post_init__(self):
        if self.path:
            self.name = Path(self.path).name
        if self.url and not urlparse(self.url).hostname:
            self.url = f"https://{self.url}"

    @property
    def raw_bytes(self) -> bytes:
        if not self.raw:
            raise ValueError(f"{self} has no raw data")
        raw = self.raw.getvalue() if isinstance(self.raw, BytesIO) else self.raw
        header = raw[:128]
        info = fleep.get(header)
        self.mimetype = info.mimes[0] if info.mimes else self.mimetype
        if info.types and info.extensions:
            self.name = f"{info.types[0]}.{info.extensions[0]}"
        return raw


@dataclass
class Image(Media):
    """Image对象, 表示一类图片元素"""

    name: str = field(default="image.png")


@dataclass
class Audio(Media):
    """Audio对象, 表示一类音频元素"""

    duration: Optional[int] = field(default=None)
    name: str = field(default="audio.mp3")


@dataclass
class Voice(Media):
    """Voice对象, 表示一类语音元素"""

    duration: Optional[int] = field(default=None)
    name: str = field(default="voice.wav")


@dataclass
class Video(Media):
    """Video对象, 表示一类视频元素"""

    name: str = field(default="video.mp4")


@dataclass
class File(Media):
    """File对象, 表示一类文件元素"""

    name: str = field(default="file.bin")


@dataclass
class Reply(Segment):
    """Reply对象，表示一类回复消息"""

    id: str
    """此处不一定是消息ID，可能是其他ID，如消息序号等"""
    msg: Optional[Union[Message, str]] = field(default=None)
    origin: Optional[Any] = field(default=None)


@dataclass
class RefNode:
    """表示转发消息的引用消息元素"""

    id: str
    context: Optional[str] = None


@dataclass
class CustomNode:
    """表示转发消息的自定义消息元素"""

    uid: str
    name: str
    time: datetime
    content: Union[str, List[Segment], Message]


@dataclass
class Reference(Segment):
    """Reference对象，表示一类引用消息。转发消息 (Forward) 也属于此类"""

    id: Optional[str] = field(default=None)
    """此处不一定是消息ID，可能是其他ID，如消息序号等"""
    content: Optional[Union[Message, str, List[Union[RefNode, CustomNode]]]] = field(default=None)


@dataclass
class Card(Segment):
    """Card对象，表示一类卡片消息"""

    flag: Literal["xml", "json"]
    raw: str
    content: Optional[Union[dict, list]] = field(default=None)

    def __post_init__(self):
        with contextlib.suppress(json.JSONDecodeError):
            self.content = json.loads(self.raw)


TM = TypeVar("TM", bound=Message)


@dataclass
class Custom(Segment, abc.ABC):
    """Custom对象，表示一类自定义消息"""

    mstype: str
    content: Any

    @abc.abstractmethod
    def export(self, msg_type: Type[TM]) -> MessageSegment[TM]: ...

    @property
    def type(self) -> str:
        return self.mstype


TCustom = TypeVar("TCustom", bound=Custom)


class _Custom(UniPattern[Custom]):
    BUILDERS: Dict[Union[str, Callable[[MessageSegment], bool]], Callable[[MessageSegment], Union[Custom, None]]] = {}

    @classmethod
    def custom_register(cls, custom_type: Type[TCustom], condition: Union[str, Callable[[MessageSegment], bool]]):
        def _register(func: Callable[[MessageSegment], Union[TCustom, None]]):
            cls.BUILDERS[condition] = func
            return func

        return _register

    def solve(self, seg: MessageSegment):
        for condition, func in self.BUILDERS.items():
            if isinstance(condition, str):
                if seg.type == condition:
                    return func(seg)
            elif condition(seg):
                return func(seg)


custom = _Custom()
custom_register = custom.custom_register


@dataclass
class Other(Segment):
    """其他 Segment"""

    origin: MessageSegment

    def __str__(self):
        return f"[{self.origin.type}]"


class _Other(UniPattern[Other]):
    def solve(self, seg: MessageSegment):
        return Other(seg)


other = _Other()


class _Text(UniPattern[Text]):
    def solve(self, seg: MessageSegment):
        if seg.type == "markdown":  # qq, console
            if "markup" in seg.data:  # console
                return Text(seg.data["markup"], "markdown")
            return Text(seg.data["content"], "markdown")
        if seg.type == "markup":  # console
            return Text(seg.data["markup"], "markup")
        if seg.type == "kmarkdown":  # kook
            return Text(seg.data["content"], "markdown")
        if seg.is_text():
            return Text(seg.data["text"], seg.type if seg.type != "text" else None)


text = _Text()


class _At(UniPattern[At]):
    def solve(self, seg: MessageSegment):
        if seg.type == "at":  # ob11, feishu, red, satori
            if "qq" in seg.data and seg.data["qq"] != "all":
                return At("user", str(seg.data["qq"]))
            if "user_id" in seg.data and seg.data["user_id"] not in ("all", "here"):
                return At("user", str(seg.data["user_id"]))
            if "id" in seg.data:
                return At("user", seg.data["id"], seg.data.get("name"))
            if "role" in seg.data:
                return At("role", seg.data["role"], seg.data.get("name"))
        if seg.type == "at_user":  # dodo
            return At("user", seg.data["dodo_id"])
        if seg.type == "at_role":  # dodo
            return At("role", seg.data["role_id"])
        if seg.type == "channel_link":  # dodo
            return At("channel", seg.data["channel_id"])
        if seg.type == "sharp":  # satori
            return At("channel", seg.data["channel_id"], seg.data["name"])
        if seg.type == "mention":
            if "user_id" in seg.data:  # ob12, kook
                return At("user", str(seg.data["user_id"]))
            if "text" in seg.data:  # tg
                return At("user", seg.data["text"])
        if seg.type == "text_mention":  # tg
            return At("user", str(seg.data["user"].id), seg.data["text"])
        if seg.type == "mention_user":
            if "user_id" in seg.data:  # qq, qqguild, discord
                return At("user", str(seg.data["user_id"]))
            if "mention_user" in seg.data:  # villa
                return At("user", str(seg.data["mention_user"].user_id), seg.data["mention_user"].user_name)
        if seg.type == "mention_channel":  # discord, qq, qqguild, kook(edited)
            return At("channel", str(seg.data["channel_id"]))
        if seg.type == "mention_role":  # discord, kook
            return At("role", str(seg.data["role_id"]))
        if seg.type == "mention_robot":  # villa
            return At("user", str(seg.data["mention_robot"].bot_id), seg.data["mention_robot"].bot_name)
        if seg.type == "At":  # mirai
            return At("user", str(seg.data["target"]), seg.data["display"])
        if seg.type == "kmarkdown":  # kook(fallback)
            content = seg.data["content"]
            if not content.startswith("(met)"):
                return None
            if (end := content.find("(met)", 5)) == -1:
                return None
            return content[5:end] not in ("here", "all") and At("user", content[5:end])
        if seg.type == "room_link":  # villa
            return At(
                "channel",
                f'{seg.data["room_link"].villa_id}:{seg.data["room_link"].room_id}',
                seg.data["room_link"].room_name,
            )


at = _At()


class _AtAll(UniPattern[AtAll]):
    def solve(self, seg: MessageSegment):
        if seg.type == "at":
            if "qq" in seg.data and seg.data["qq"] == "all":  # ob11
                return AtAll()
            if "user_id" in seg.data and seg.data["user_id"] in ("all", "here"):  # feishu
                return AtAll()
            if "type" in seg.data and seg.data["type"] in ("all", "here"):  # satori
                return AtAll(here=seg.data["type"] == "here")
        if seg.type in {"at_all", "AtAll", "mention_everyone", "mention_all"}:
            return AtAll()
        if seg.type == "mention_here":  # kook
            return AtAll(here=True)
        if seg.type == "kmarkdown":  # kook(fallback)
            content = seg.data["content"]
            if not content.startswith("(met)"):
                return None
            if (end := content.find("(met)", 5)) == -1:
                return None
            return content[5:end] in ("here", "all") and AtAll(content[5:end] == "here")


at_all = _AtAll()


class _Emoji(UniPattern[Emoji]):
    def solve(self, seg: MessageSegment):
        if seg.type == "emoji":
            if "id" in seg.data:
                if "name" in seg.data:
                    return Emoji(seg.data["id"], seg.data["name"])
                return Emoji(seg.data["id"])
            if "name" in seg.data:
                return Emoji(seg.data["name"])
        if seg.type == "Face":
            return Emoji(str(seg.data["faceId"]), seg.data["name"])
        if seg.type == "face":
            if "id" in seg.data:
                return Emoji(str(seg.data["id"]))
            if "face_id" in seg.data:
                return Emoji(str(seg.data["face_id"]))
        if seg.type == "custom_emoji":
            if "custom_emoji_id" in seg.data:  # telegram
                return Emoji(seg.data["custom_emoji_id"], seg.data["text"])
            if "id" in seg.data:  # discord
                return Emoji(seg.data["id"], seg.data["name"])
        if seg.type == "kmarkdown":  # kook(fallback)
            content = seg.data["content"]
            if content.startswith("(emj)"):
                mat = re.search(r"\(emj\)(?P<name>[^()\[\]]+)\(emj\)\[(?P<id>[^\[\]]+)\]", content)
                return mat and Emoji(mat["id"], mat["name"])
            if content.startswith(":"):
                mat = re.search(r":(?P<id>[^:]+):", content)
                return mat and Emoji(mat["id"])
        if seg.type == "sticker" and "id" in seg.data:
            return Emoji(seg.data["id"])


emoji = _Emoji()


class _Image(UniPattern[Image]):
    def solve(self, seg: MessageSegment):
        if seg.type == "image":
            if "uuid" in seg.data:  # red
                return Image(
                    id=seg.data["uuid"],
                    path=seg.data["path"],
                    name=seg.data["md5"],
                )
            if "file_id" in seg.data:  # ob12
                return Image(id=seg.data["file_id"])
            if "image" in seg.data:  # villa
                return Image(url=seg.data["image"].url)
            if "image_key" in seg.data:  # feishu
                return Image(id=seg.data["image_key"])
            if "file_key" in seg.data:  # kook
                return Image(url=seg.data["file_key"])
            if "url" in seg.data:  # ob11, qq
                if "filename" in seg.data:
                    return Image(
                        url=seg.data["url"],
                        id=seg.data["filename"],
                        mimetype=seg.data["content_type"],
                        name=seg.data["filename"],
                    )
                if "file" in seg.data:
                    return Image(url=seg.data["url"], id=seg.data["file"])
                return Image(url=seg.data["url"])
            if "file_path" in seg.data:  # ntchat
                return Image(id=seg.data["file_path"], path=seg.data["file_path"])
            if "picURL" in seg.data:  # ding
                return Image(url=seg.data["picURL"])
        if seg.type == "photo":  # tg
            return Image(id=seg.data["file"])
        if seg.type == "attachment":
            if "url" in seg.data:  # qqguild
                return Image(url=seg.data["url"])
            if "attachment" in seg.data:  # discord
                return Image(id=seg.data["attachment"].filename)
        if seg.type == "picture":  # dodo
            return Image(url=seg.data["picture"].url)
        if seg.type == "Image":  # mirai
            return Image(url=seg.data["url"], id=seg.data["imageId"])
        if seg.type == "img":  # satori
            src = seg.data["src"]
            if src.startswith("http"):
                return Image(url=src)
            if src.startswith("file://"):
                return Image(path=Path(src[7:]))
            if src.startswith("data:"):
                mime, b64 = src[5:].split(";", 1)
                return Image(raw=b64decode(b64[7:]), mimetype=mime)
            return Image(seg.data["src"])


image = _Image()


class _Video(UniPattern[Video]):
    def solve(self, seg: MessageSegment):
        if seg.type == "video":
            if "videoMd5" in seg.data:  # red
                return Video(
                    id=seg.data["videoMd5"],
                    path=seg.data["filePath"],
                    name=seg.data["fileName"],
                )
            if "video" in seg.data:  # dodo
                return Video(url=seg.data["video"].url)
            if "file_id" in seg.data:  # ob12, telegram
                return Video(id=seg.data["file_id"])
            if "file" in seg.data:  # ob11
                return Video(url=seg.data["file"])
            if "url" in seg.data:  # qq
                if "filename" in seg.data:
                    return Video(
                        url=seg.data["url"],
                        id=seg.data["filename"],
                        mimetype=seg.data["content_type"],
                        name=seg.data["filename"],
                    )
                return Video(url=seg.data["url"])
            if "file_key" in seg.data:  # kook
                return Video(url=seg.data["file_key"])
            if "file_path" in seg.data:  # ntchat
                return Video(id=seg.data["file_path"], path=seg.data["file_path"])
            if "src" in seg.data:  # satori
                src = seg.data["src"]
                if src.startswith("http"):
                    return Video(url=src)
                if src.startswith("file://"):
                    return Video(path=Path(src[7:]))
                if src.startswith("data:"):
                    mime, b64 = src[5:].split(";", 1)
                    return Video(raw=b64decode(b64[7:]), mimetype=mime)
                return Video(seg.data["src"])
        if seg.type == "Video":  # mirai
            return Video(url=seg.data["url"], id=seg.data["videoId"])
        if seg.type == "animation":  # telegram
            return Video(id=seg.data["file_id"])
        if seg.type == "media":  # feishu
            return Video(id=seg.data["file_key"], name=seg.data["file_name"])


video = _Video()


class _Voice(UniPattern[Voice]):
    def solve(self, seg: MessageSegment):
        if seg.type == "voice":
            if "md5" in seg.data:  # red
                return Voice(
                    id=seg.data["md5"],
                    path=seg.data["path"],
                    name=seg.data["name"],
                )
            if "file_id" in seg.data:  # ob12, telegram
                return Voice(id=seg.data["file_id"])
            if "file_key" in seg.data:  # kook
                return Voice(url=seg.data["file_key"])
            if "file_path" in seg.data:  # ntchat
                return Voice(id=seg.data["file_path"], path=seg.data["file_path"])
        if seg.type == "record":  # ob11
            return Voice(url=seg.data["url"])
        if seg.type == "Voice":  # mirai
            return Voice(url=seg.data["url"], id=seg.data["voiceId"])


voice = _Voice()


class _Audio(UniPattern[Audio]):
    def solve(self, seg: MessageSegment):
        if seg.type != "audio":
            return
        if "url" in seg.data:  # qq
            if "filename" in seg.data:
                return Audio(
                    url=seg.data["url"],
                    id=seg.data["filename"],
                    mimetype=seg.data["content_type"],
                    name=seg.data["filename"],
                )
            return Audio(url=seg.data["url"])
        if "file_id" in seg.data:  # ob12, telegram
            return Audio(id=seg.data["file_id"])
        if "file_key" in seg.data:  # kook, feishu
            return Audio(url=seg.data["file_key"])
        if "file_path" in seg.data:  # ntchat
            return Audio(id=seg.data["file_path"], path=seg.data["file_path"])
        if "src" in seg.data:  # satori
            src = seg.data["src"]
            if src.startswith("http"):
                return Audio(url=src)
            if src.startswith("file://"):
                return Audio(path=Path(src[7:]))
            if src.startswith("data:"):
                mime, b64 = src[5:].split(";", 1)
                return Audio(raw=b64decode(b64[7:]), mimetype=mime)
            return Audio(seg.data["src"])


audio = _Audio()


class _File(UniPattern[File]):
    def solve(self, seg: MessageSegment):
        if seg.type == "file":
            if "md5" in seg.data:  # red
                return File(
                    id=seg.data["md5"],
                    name=seg.data["name"],
                )
            if "url" in seg.data:  # qq
                if "filename" in seg.data:
                    return File(
                        url=seg.data["url"],
                        id=seg.data["filename"],
                        mimetype=seg.data["content_type"],
                        name=seg.data["filename"],
                    )
                return File(url=seg.data["url"])
            if "file" in seg.data:  # dodo
                return File(url=seg.data["file"].url)
            if "file_id" in seg.data:  # ob12
                return File(id=seg.data["file_id"])
            if "file_key" in seg.data:  # feishu, kook
                return File(
                    id=seg.data["file_key"],
                    name=seg.data.get("file_name", seg.data.get("title")),
                )
            if "file_path" in seg.data:  # ntchat
                return File(id=seg.data["file_path"])
            if "src" in seg.data:  # satori
                src = seg.data["src"]
                if src.startswith("http"):
                    return File(url=src)
                if src.startswith("file://"):
                    return File(path=Path(src[7:]))
                if src.startswith("data:"):
                    mime, b64 = src[5:].split(";", 1)
                    return File(raw=b64decode(b64[7:]), mimetype=mime)
                return File(seg.data["src"])
        if seg.type == "document":  # telegram
            return File(seg.data["file_id"], name=seg.data["file_name"])
        if seg.type == "File":  # mirai
            return File(seg.data["id"], name=seg.data["name"])


file = _File()


class _Reply(UniPattern[Reply]):
    def solve(self, seg: MessageSegment):
        if seg.type == "reference":
            if "message_id" in seg.data:  # telegram, dodo
                return Reply(seg.data["message_id"], origin=seg)
            if "reference" in seg.data:  # discord, qq, qqguild
                return Reply(seg.data["reference"].message_id, origin=seg.data["reference"])
        if seg.type == "reply":
            if "id" in seg.data:  # ob11
                return Reply(seg.data["id"], origin=seg)
            if "message_id" in seg.data:  # ob12, telegram
                return Reply(seg.data["message_id"], origin=seg)
            if "msg_id" in seg.data:  # red
                return Reply(seg.data["msg_seq"], origin=seg.data["_origin"])
        if seg.type == "quote":
            if "id" in seg.data:  # satori
                return Reply(seg.data["id"], seg.data.get("content"), seg)
            if "msg_id" in seg.data:  # kook:
                return Reply(seg.data["msg_id"], origin=seg)
            if "quoted_message_id" in seg.data:  # villa
                return Reply(seg.data["quote"].quoted_message_id, origin=seg.data["quote"])
        if seg.type == "Quote":  # mirai
            return Reply(str(seg.data["id"]), seg.data["origin"], origin=seg)


reply = _Reply()


class _Reference(UniPattern[Reference]):
    def solve(self, seg: MessageSegment):
        if seg.type == "post":  # villa
            return Reference(seg.data["post"].post_id)
        if seg.type == "message":  # satori
            return Reference(seg.data.get("id"), seg.data.get("content"))
        if seg.type == "forward":
            if "xml" in seg.data:  # red
                return Reference(seg.data["id"], seg.data["xml"])
            return Reference(seg.data["id"])  # ob11
        if seg.type == "Forward":  # mirai
            nodes = []
            for node in seg.data["nodeList"]:
                if "messageId" in node:
                    nodes.append(RefNode(node["messageId"]))
                elif "messageRef" in node:
                    nodes.append(RefNode(node["messageRef"]["messageId"], node["messageRef"]["target"]))
                else:
                    nodes.append(CustomNode(node["senderId"], node["senderName"], node["time"], node["messageChain"]))
            return Reference(seg.data.get("messageId"), nodes)


reference = _Reference()


class _Card(UniPattern[Card]):
    def solve(self, seg: MessageSegment):
        if seg.type == "card":
            if "content" in seg.data:  # kook
                return Card("json", seg.data["content"])
            if "card_wxid" in seg.data:  # ntchat
                return Card("json", seg.data["card_wxid"])
        if seg.type == "Xml":  # mirai
            return Card("xml", seg.data["xml"])
        if seg.type == "Json":  # mirai
            return Card("json", seg.data["json"])
        if seg.type == "App":  # mirai
            return Card("json", seg.data["content"])
        if seg.type == "xml":  # ob12
            return Card("xml", seg.data["data"])
        if seg.type == "json":  # ob11
            return Card("json", seg.data["data"])
        if seg.type == "ark" and "data" in seg.data:  # red
            return Card("json", seg.data["data"])


card = _Card()
segments = [at_all, at, emoji, image, video, voice, audio, file, reference, card, text, custom, other]
env = create_local_patterns("nonebot")
env.sets(segments)


class _Segment(UniPattern[Segment]):
    def solve(self, seg: MessageSegment):
        for pat in segments:
            if (res := pat.validate(seg)).success:
                res._value.origin = seg
                return res._value
        return Other(seg)  # type: ignore


env[Segment] = _Segment()


async def reply_handle(event: Event, bot: Bot):
    adapter = bot.adapter
    adapter_name = adapter.get_name()
    if adapter_name == "Telegram":
        if TYPE_CHECKING:
            from nonebot.adapters.telegram.event import MessageEvent

            assert isinstance(event, MessageEvent)
        if event.reply_to_message:
            return Reply(
                f"{event.reply_to_message.message_id}",
                event.reply_to_message.original_message,
                event.reply_to_message,
            )
    elif adapter_name == "Feishu":
        if TYPE_CHECKING:
            from nonebot.adapters.feishu.event import MessageEvent

            assert isinstance(event, MessageEvent)
        if event.reply:
            return Reply(event.reply.message_id, event.reply.body.content, event.reply)
    elif adapter_name == "ntchat":
        if TYPE_CHECKING:
            from nonebot.adapters.ntchat.event import QuoteMessageEvent

            assert isinstance(event, QuoteMessageEvent)
        if event.type == 11061:
            return Reply(event.quote_message_id, origin=event)
    elif adapter_name == "QQ Guild":
        if TYPE_CHECKING:
            from nonebot.adapters.qqguild.event import MessageEvent

            assert isinstance(event, MessageEvent)
        if event.reply and event.reply.message:
            return Reply(
                str(event.reply.message.id),
                event.reply.message.content,
                event.reply.message,
            )
    elif adapter_name == "QQ":
        if TYPE_CHECKING:
            from nonebot.adapters.qq.models import Message as GuildMessage
            from nonebot.adapters.qq.event import QQMessageEvent, GuildMessageEvent

            assert isinstance(event, (GuildMessageEvent, QQMessageEvent))
        if rpl := getattr(event, "reply", None):
            if TYPE_CHECKING:
                assert isinstance(rpl, GuildMessage)
            return Reply(
                str(rpl.id),
                rpl.content,
                rpl,
            )
    elif adapter_name == "Satori":
        if TYPE_CHECKING:
            from nonebot.adapters.satori.event import MessageEvent

            assert isinstance(event, MessageEvent)
        if event.reply:
            return Reply(
                str(event.reply.data.get("id")),
                event.reply.data.get("content"),
                event.reply,
            )
    elif adapter_name == "mirai2":
        if TYPE_CHECKING:
            from nonebot.adapters.mirai2.event import MessageEvent

            assert isinstance(event, MessageEvent)
        if event.quote:
            return Reply(str(event.quote.id), event.quote.origin, event.quote)
    elif adapter_name == "Kaiheila":
        if TYPE_CHECKING:
            from nonebot.adapters.kaiheila import Bot as KaiheilaBot
            from nonebot.adapters.kaiheila.event import MessageEvent

            assert isinstance(event, MessageEvent)
            assert isinstance(bot, KaiheilaBot)

        api = "directMessage_view" if event.__event__ == "message.private" else "message_view"
        message = await bot.call_api(
            api,
            msg_id=event.msg_id,
            **({"chat_code": event.event.code} if event.__event__ == "message.private" else {}),
        )
        if message.quote:
            return Reply(message.quote.id_, origin=message.quote)
    elif adapter_name == "Discord":
        if TYPE_CHECKING:
            from nonebot.adapters.discord import MessageEvent

            assert isinstance(event, MessageEvent)

        if hasattr(event, "message_reference") and hasattr(event.message_reference, "message_id"):
            return Reply(
                event.message_reference.message_id,  # type: ignore
                origin=event.message_reference,  # type: ignore
            )
    elif adapter_name == "RedProtocol":
        if TYPE_CHECKING:
            from nonebot.adapters.red.event import MessageEvent

            assert isinstance(event, MessageEvent)

        if event.reply:
            return Reply(
                f"{event.reply.sourceMsgIdInRecords}#{event.reply.replayMsgSeq}",
                event.reply.sourceMsgTextElems,
                origin=event.reply,
            )
    elif adapter_name == "Villa":
        if TYPE_CHECKING:
            from nonebot.adapters.villa.event import SendMessageEvent

            assert isinstance(event, SendMessageEvent)

        if event.quote_msg:
            return Reply(
                f"{event.quote_msg.msg_uid}@{event.quote_msg.send_at}",
                msg=event.quote_msg.content,
                origin=event.quote_msg,
            )
    elif _reply := getattr(event, "reply", None):
        return Reply(str(_reply.message_id), getattr(_reply, "message", None), _reply)
    return None



import inspect
from typing_extensions import Annotated, get_args
from typing import Any, Dict, Type, Tuple, Union, Literal, TypeVar, ClassVar, Optional, overload

from nonebot.typing import T_State
from tarina import run_always_await
from nepattern.util import CUnionType
from tarina.generic import get_origin
from nonebot.dependencies import Param
from nonebot.internal.params import Depends
from nonebot.compat import PydanticUndefined
from nonebot.internal.matcher import Matcher
from nonebot.internal.adapter import Bot, Event
from arclet.alconna.builtin import generate_duplication
from arclet.alconna import Empty, Alconna, Arparma, Duplication

from .typings import CHECK, MIDDLEWARE
from .model import T, Match, Query, CommandResult
from .extension import Extension, ExtensionExecutor
from .consts import ALCONNA_RESULT, ALCONNA_ARG_KEY, ALCONNA_EXTENSION, ALCONNA_EXEC_RESULT

T_Duplication = TypeVar("T_Duplication", bound=Duplication)
T_Extension = TypeVar("T_Extension", bound=Extension)

_Contents = (Union, CUnionType, Literal)


def _alconna_result(state: T_State) -> CommandResult:
    return state[ALCONNA_RESULT]


def AlconnaResult() -> CommandResult:
    return Depends(_alconna_result, use_cache=False)


def _alconna_exec_result(state: T_State) -> Dict[str, Any]:
    return state[ALCONNA_EXEC_RESULT]


def AlconnaExecResult() -> Dict[str, Any]:
    return Depends(_alconna_exec_result, use_cache=False)


def _alconna_matches(state: T_State) -> Arparma:
    return _alconna_result(state).result


def AlconnaMatches() -> Arparma:
    return Depends(_alconna_matches, use_cache=False)


def _alconna_ctx(state: T_State):
    return state[ALCONNA_RESULT].context


def AlconnaContext() -> Dict[str, Any]:
    return Depends(_alconna_ctx, use_cache=False)


def AlconnaMatch(name: str, middleware: Optional[MIDDLEWARE] = None) -> Match:
    async def _alconna_match(state: T_State, bot: Bot, event: Event) -> Match:
        arp = _alconna_result(state).result
        mat = Match(arp.all_matched_args.get(name, Empty), name in arp.all_matched_args)
        if middleware and mat.available:
            mat.result = await run_always_await(middleware, event, bot, state, mat.result)
        return mat

    return Depends(_alconna_match, use_cache=False)


def merge_path(path: str, parent: str) -> str:
    if not path.startswith("~"):
        return path
    if path.startswith("~."):
        path = path.replace("~.", "~", 1)
    return f"{parent}.{path[1:]}" if parent else path[1:]


def AlconnaQuery(
    path: str,
    default: Union[T, Empty] = Empty,
    middleware: Optional[MIDDLEWARE] = None,
) -> Query[T]:
    async def _alconna_query(state: T_State, bot: Bot, event: Event, matcher: Matcher) -> Query:
        arp = _alconna_result(state).result
        _path = merge_path(path, getattr(matcher, "basepath", ""))
        q = Query(_path, default)
        result = arp.query(_path, Empty)
        q.available = result != Empty
        if q.available:
            q.result = result  # type: ignore
        elif default != Empty:
            q.available = True
        if middleware and q.available:
            q.result = await run_always_await(middleware, event, bot, state, q.result)
        return q

    return Depends(_alconna_query, use_cache=False)


@overload
def AlconnaDuplication() -> Duplication: ...


@overload
def AlconnaDuplication(__t: Type[T_Duplication]) -> T_Duplication: ...


def AlconnaDuplication(__t: Optional[Type[T_Duplication]] = None) -> Duplication:
    def _alconna_match(state: T_State) -> Duplication:
        res = _alconna_result(state)
        gt = __t or generate_duplication(res.source)
        return gt(res.result)

    return Depends(_alconna_match, use_cache=False)


def AlconnaArg(path: str) -> Any:
    def _alconna_arg(state: T_State) -> Any:
        return state[ALCONNA_ARG_KEY.format(key=path)]

    return Depends(_alconna_arg, use_cache=False)


AlcResult = Annotated[CommandResult, AlconnaResult()]
AlcExecResult = Annotated[Dict[str, Any], AlconnaExecResult()]
AlcMatches = Annotated[Arparma, AlconnaMatches()]
AlcContext = Annotated[Dict[str, Any], AlconnaContext()]


def match_path(
    path: str,
    additional: Optional[CHECK] = None,
):
    """
    当 Arpamar 解析成功后, 依据 path 是否存在以继续执行事件处理

    当 path 为 ‘$main’ 时表示认定当且仅当主命令匹配
    """

    async def wrapper(event: Event, bot: Bot, state: T_State, result: Arparma):
        if path == "$main":
            return not result.components and (not additional or await additional(event, bot, state, result))
        else:
            return result.query(path, "\0") != "\0" and (not additional or await additional(event, bot, state, result))

    return wrapper


def match_value(
    path: str,
    value: Any,
    or_not: bool = False,
    additional: Optional[CHECK] = None,
):
    """
    当 Arpamar 解析成功后, 依据查询 path 得到的结果是否符合传入的值以继续执行事件处理

    当 or_not 为真时允许查询 path 失败时继续执行事件处理
    """

    async def wrapper(event: Event, bot: Bot, state: T_State, result: Arparma):
        if result.query(path, "\0") == value:
            return True and (not additional or await additional(event, bot, state, result))
        return (
            or_not
            and result.query(path, "\0") == "\0"
            and (not additional or await additional(event, bot, state, result))
        )

    return wrapper


_seminal = type("_seminal", (object,), {})


def assign(
    path: str,
    value: Any = _seminal,
    or_not: bool = False,
    additional: Optional[CHECK] = None,
) -> CHECK:
    if value != _seminal:
        return match_value(path, value, or_not, additional)
    if or_not:

        async def wrapper(event: Event, bot: Bot, state: T_State, result: Arparma):
            return await match_path("$main", additional)(event, bot, state, result) or await match_path(
                path, additional
            )(event, bot, state, result)

        return wrapper
    return match_path(path, additional)


def Check(fn: CHECK) -> bool:
    async def _arparma_check(bot: Bot, state: T_State, event: Event, matcher: Matcher) -> bool:
        arp = _alconna_result(state).result
        if not (ans := await fn(event, bot, state, arp)):
            matcher.skip()
        return ans

    return Depends(_arparma_check, use_cache=False)


def AlconnaExtension(target: Type[T_Extension]) -> T_Extension:
    def _alconna_extension(state: T_State):
        exts = state[ALCONNA_EXTENSION]
        return next((i for i in exts if isinstance(i, target)), None)  # type: ignore

    return Depends(_alconna_extension, use_cache=False)


class ExtensionParam(Param):
    """Extension 自定义注入参数"""

    executor: ClassVar[ExtensionExecutor]

    def __repr__(self) -> str:
        return f"ExtensionParam(name={self.extra['name']}, type={self.extra['type']!r})"

    @classmethod
    def new(cls, executor: ExtensionExecutor):
        return type(
            f"ExtensionParam_{executor._rule.command.name}",
            (cls,),
            {"executor": executor},
        )

    @classmethod
    def _check_param(cls, param: inspect.Parameter, allow_types: Tuple[Type[Param], ...]) -> Optional["ExtensionParam"]:
        if cls.executor.before_catch(param.name, param.annotation, param.default):
            return cls(param.default, name=param.name, type=param.annotation, validate=True)

    async def _solve(self, matcher: Matcher, event: Event, state: T_State, **kwargs: Any) -> Any:
        res = await self.executor.catch(event, state, self.extra["name"], self.extra["type"], self.default)
        if res is not PydanticUndefined:
            return res
        return self.default


class AlconnaParam(Param):
    """Alconna 相关注入参数

    本注入解析事件响应器操作 `AlconnaMatcher` 的响应函数内所需参数。
    """

    def __repr__(self) -> str:
        return f"AlconnaParam(type={self.extra['type']!r})"

    @classmethod
    def _check_param(cls, param: inspect.Parameter, allow_types: Tuple[Type[Param], ...]) -> Optional["AlconnaParam"]:
        annotation = get_origin(param.annotation)
        if annotation in _Contents:
            annotation = get_args(param.annotation)[0]
        if annotation is CommandResult:
            return cls(..., type=CommandResult)
        if annotation is Arparma:
            return cls(..., type=Arparma)
        if annotation is Alconna:
            return cls(..., type=Alconna)
        if annotation is Duplication:
            return cls(..., type=Duplication)
        if inspect.isclass(annotation) and issubclass(annotation, Duplication):
            return cls(..., anno=param.annotation, type=Duplication)
        if inspect.isclass(annotation) and issubclass(annotation, Extension):
            return cls(..., anno=param.annotation, type=Extension)
        if annotation is Match:
            return cls(param.default, name=param.name, type=Match)
        if isinstance(param.default, Query):
            return cls(param.default, type=Query)
        if param.name in ("ctx", "context") and annotation is dict:
            return cls(..., type=Literal["context"])
        return cls(param.default, name=param.name, type=param.annotation, validate=True)

    async def _solve(self, matcher: Matcher, event: Event, state: T_State, **kwargs: Any) -> Any:
        t = self.extra["type"]
        if ALCONNA_RESULT not in state:
            return self.default if self.default not in (..., Empty) else PydanticUndefined
        res: CommandResult = state[ALCONNA_RESULT]
        if t is CommandResult:
            return res
        if t is Arparma:
            return res.result
        if t is Alconna:
            return res.source
        if t is Duplication:
            if anno := self.extra.get("anno"):
                return anno(res.result)
            else:
                return generate_duplication(res.source)(res.result)
        if t is Extension:
            anno = self.extra["anno"]
            return next((i for i in state[ALCONNA_EXTENSION] if isinstance(i, anno)), None)  # type: ignore
        if t is Match:
            target = res.result.all_matched_args.get(self.extra["name"], Empty)
            return Match(target, target != Empty)
        if t is Query:
            _path = merge_path(self.default.path, getattr(matcher, "basepath", ""))
            q = Query(_path, self.default.result)
            result = res.result.query(q.path, Empty)
            q.available = result != Empty
            if q.available:
                q.result = result
            elif self.default.result != Empty:
                q.available = True
            return q
        if t == Literal["context"]:
            return res.result.context
        if (key := ALCONNA_ARG_KEY.format(key=self.extra["name"])) in state:
            return state[key]
        if self.extra["name"] in res.result.all_matched_args:
            return res.result.all_matched_args[self.extra["name"]]
        return self.default if self.default not in (..., Empty) else PydanticUndefined

    async def _check(self, state: T_State, **kwargs: Any) -> Any:
        if self.extra["type"] == Any:
            if (
                self.extra["name"] in _alconna_result(state).result.all_matched_args
                or ALCONNA_ARG_KEY.format(key=self.extra["name"]) in state
            ):
                return True
            if self.default not in (..., Empty):
                return True


class _Dispatch:
    def __init__(
        self,
        path: str,
        value: Any = _seminal,
        or_not: bool = False,
        additional: Optional[CHECK] = None,
    ):
        self.fn = assign(path, value, or_not, additional)
        self.result = None

    def set(self, arp: AlcResult):
        self.result = arp

    async def __call__(self, _state: T_State, event: Event, bot: Bot) -> bool:
        if self.result is None:
            return False
        if await self.fn(event, bot, _state, self.result.result):
            _state[ALCONNA_RESULT] = self.result
            self.result = None
            return True
        return False



from typing import TYPE_CHECKING, Any, Union, cast

from tarina import lang
from nonebot.adapters import Bot, Event, Message

from ..export import Target, SupportAdapter, MessageExporter, SerializeFailed, export
from ..segment import At, File, Text, AtAll, Audio, Image, Reply, Video, Voice, RefNode, Reference

if TYPE_CHECKING:
    from nonebot.adapters.satori.message import MessageSegment


class SatoriMessageExporter(MessageExporter["MessageSegment"]):
    def get_message_type(self):
        from nonebot.adapters.satori.message import Message

        return Message

    @classmethod
    def get_adapter(cls) -> SupportAdapter:
        return SupportAdapter.satori

    def get_target(self, event: Event, bot: Union[Bot, None] = None) -> Target:
        from nonebot.adapters.satori.event import NoticeEvent, MessageEvent

        if isinstance(event, (MessageEvent, NoticeEvent)):
            if event.channel:
                return Target(
                    event.channel.id,
                    event.guild.id if event.guild else event.channel.parent_id or "",
                    platform=self.get_adapter(),
                    self_id=bot.self_id if bot else None,
                )
            elif event.user:
                return Target(
                    event.user.id,
                    private=True,
                    platform=self.get_adapter(),
                    self_id=bot.self_id if bot else None,
                )
        raise NotImplementedError

    def get_message_id(self, event: Event) -> str:
        from nonebot.adapters.satori.event import MessageEvent

        assert isinstance(event, MessageEvent)
        return str(event.message.id)

    @export
    async def text(self, seg: Text, bot: Bot) -> "MessageSegment":
        from nonebot.adapters.satori.message import STYLE_TYPE_MAP

        ms = self.segment_class

        if not seg.style:
            return ms.text(seg.text)
        if seg.style == "br" or seg.text == "\n":
            return ms.br()
        if seg.style in STYLE_TYPE_MAP:
            seg_cls, seg_type = STYLE_TYPE_MAP[seg.style]
            return seg_cls(seg_type, {"text": seg.text})
        if hasattr(ms, seg.style):
            return getattr(ms, seg.style)(seg.text)
        raise SerializeFailed(lang.require("nbp-uniseg", "invalid_segment").format(type=seg.style, seg=seg))

    @export
    async def at(self, seg: At, bot: Bot) -> "MessageSegment":
        ms = self.segment_class
        if seg.flag == "role":
            return ms.at_role(seg.target, seg.display)
        if seg.flag == "channel":
            return ms.sharp(seg.target, seg.display)
        return ms.at(seg.target, seg.display)

    @export
    async def at_all(self, seg: AtAll, bot: Bot) -> "MessageSegment":
        ms = self.segment_class
        return ms.at_all(seg.here)

    @export
    async def res(self, seg: Union[Image, Voice, Video, Audio, File], bot: Bot) -> "MessageSegment":
        ms = self.segment_class

        name = seg.__class__.__name__.lower()
        method = {
            "image": ms.image,
            "voice": ms.audio,
            "video": ms.video,
            "audio": ms.audio,
            "file": ms.file,
        }[name]
        if seg.id or seg.url:
            return method(url=seg.id or seg.url)
        if seg.path:
            return method(path=seg.path)
        if seg.raw:
            data = seg.raw_bytes
            if seg.mimetype:
                return method(raw={"data": data, "mime": seg.mimetype})
        raise SerializeFailed(lang.require("nbp-uniseg", "invalid_segment").format(type=name, seg=seg))

    @export
    async def reply(self, seg: Reply, bot: Bot) -> "MessageSegment":
        ms = self.segment_class

        return ms.quote(seg.id, content=seg.msg)  # type: ignore

    @export
    async def reference(self, seg: Reference, bot: Bot) -> "MessageSegment":
        ms = self.segment_class
        if isinstance(seg.content, str):
            content = self.get_message_type()(seg.content)
        elif isinstance(seg.content, list):
            content = self.get_message_type()()
            for node in seg.content:
                if isinstance(node, RefNode):
                    content.append(ms.message(node.id))
                else:
                    _content = self.get_message_type()()
                    _content.append(ms.author(node.uid, node.name))
                    if isinstance(node.content, str):
                        _content.extend(self.get_message_type()(node.content))
                    elif isinstance(node.content, list):
                        _content.extend(await self.export(node.content, bot, True))  # type: ignore
                    else:
                        _content.extend(node.content)
                    content.append(ms.message(content=_content))
        else:
            content = seg.content
        return ms.message(seg.id, bool(seg.content), content)

    async def send_to(self, target: Union[Target, Event], bot: Bot, message: Message):
        from nonebot.adapters.satori.bot import Bot as SatoriBot

        assert isinstance(bot, SatoriBot)
        if TYPE_CHECKING:
            assert isinstance(message, self.get_message_type())

        if isinstance(target, Event):
            target = self.get_target(target, bot)

        if target.private:
            return await bot.send_private_message(target.id, message)
        else:
            return await bot.send_message(target.id, message)

    async def recall(self, mid: Any, bot: Bot, context: Union[Target, Event]):
        from nonebot.adapters.satori.models import InnerMessage
        from nonebot.adapters.satori.bot import Bot as SatoriBot

        assert isinstance(bot, SatoriBot)
        _mid: InnerMessage = cast(InnerMessage, mid)
        if isinstance(context, Target):
            if context.private:
                channel = await bot.user_channel_create(user_id=context.id)
                await bot.message_delete(channel_id=channel.id, message_id=_mid.id)
            else:
                await bot.message_delete(channel_id=context.id, message_id=_mid.id)
        else:
            channel = _mid.channel or context.channel  # type: ignore
            await bot.message_delete(channel_id=channel.id, message_id=_mid.id)
        return

    async def edit(self, new: Message, mid: Any, bot: Bot, context: Union[Target, Event]):
        from nonebot.adapters.satori.models import InnerMessage
        from nonebot.adapters.satori.bot import Bot as SatoriBot

        assert isinstance(bot, SatoriBot)
        if TYPE_CHECKING:
            assert isinstance(new, self.get_message_type())

        _mid: InnerMessage = cast(InnerMessage, mid)
        if isinstance(context, Target):
            if context.private:
                channel = await bot.user_channel_create(user_id=context.id)
                return await bot.update_message(channel.id, _mid.id, new)
            return await bot.update_message(context.id, _mid.id, new)
        channel = mid.channel or context.channel  # type: ignore
        return await bot.update_message(channel.id, _mid.id, new)

    def get_reply(self, mid: Any):
        from nonebot.adapters.satori.models import InnerMessage

        _mid: InnerMessage = cast(InnerMessage, mid)
        return Reply(_mid.id)



from arclet.alconna import Args, Alconna


def test_v11():
    from nonebot.adapters.onebot.v11 import Message

    from nonebot_plugin_alconna.adapters.onebot11 import At, AtID, Face

    msg = Message("Hello!11") + At(123)
    msg1 = Message("Hello!11 @123")
    msg2 = Message("Hello!11 123")

    alc = Alconna("Hello!11", Args["target", AtID])
    assert alc.parse(msg).matched
    assert alc.parse(msg1).matched
    assert alc.parse(msg2).matched
    assert not alc.parse(Message("Hello!11 @abcd")).matched
    assert not alc.parse(Message("Hello!11") + Face(123)).matched


def test_v12():
    from nonebot.adapters.onebot.v12 import Message

    from nonebot_plugin_alconna.adapters.onebot12 import Image, Mention, MentionID

    msg = Message("Hello!12") + Mention("123")
    msg1 = Message("Hello!12 @123")
    msg2 = Message("Hello!12 123")

    alc = Alconna("Hello!12", Args["target", MentionID])
    assert alc.parse(msg).matched
    assert alc.parse(msg1).matched
    assert alc.parse(msg2).matched
    assert not alc.parse(Message("Hello!12 @abcd")).matched
    assert not alc.parse(Message("Hello!12") + Image("1.png")).matched



from typing import Union, cast

from nonebot.adapters import Bot, Event, Message, MessageSegment

from ..segment import Text
from ..export import Target, SupportAdapter, MessageExporter, export


class BilibiliMessageExporter(MessageExporter):
    def get_message_type(self):
        from nonebot.adapters.bilibili.message import Message  # type: ignore

        return Message

    @classmethod
    def get_adapter(cls) -> SupportAdapter:
        return SupportAdapter.bilibili

    def get_message_id(self, event: Event) -> str:
        from nonebot.adapters.bilibili.event import MessageEvent  # type: ignore

        assert isinstance(event, MessageEvent)
        return str(event.session_id)  # type: ignore

    @export
    async def text(self, seg: Text, bot: Bot) -> MessageSegment:
        msg = self.get_message_type()
        ms = msg.get_segment_class()  # type: ignore

        return ms.danmu(seg.text)

    async def send_to(self, target: Union[Target, Event], bot: Bot, message: Message):
        from nonebot.adapters.bilibili.adapter import Adapter  # type: ignore

        adapter: Adapter = cast(Adapter, bot.adapter)
        return await adapter.bili.send(str(message), bot.self_id)



from typing import TYPE_CHECKING, Union

from nonebot.adapters import Bot, Event, Message

from ..segment import Text
from ..export import Target, SupportAdapter, MessageExporter, export

if TYPE_CHECKING:
    from nonebot.adapters.minecraft.message import MessageSegment


class MinecraftMessageExporter(MessageExporter["MessageSegment"]):
    def get_message_type(self):
        from nonebot.adapters.minecraft.message import Message

        return Message

    @classmethod
    def get_adapter(cls) -> SupportAdapter:
        return SupportAdapter.minecraft

    def get_message_id(self, event: Event) -> str:
        from nonebot.adapters.minecraft.event.base import MessageEvent

        assert isinstance(event, MessageEvent)
        return str(id(event))

    @export
    async def text(self, seg: Text, bot: Bot) -> "MessageSegment":
        ms = self.segment_class
        return ms.text(seg.text)

    async def send_to(self, target: Union[Target, Event], bot: Bot, message: Message):
        from nonebot.adapters.minecraft.bot import Bot as MinecraftBot

        assert isinstance(bot, MinecraftBot)

        return await bot.send_msg(message=message)  # type: ignore



import re

from nonebot.adapters.kaiheila.message import Message
from nonebot.adapters.kaiheila.message import Card as _Card
from nonebot.adapters.kaiheila.message import File as _File
from nonebot.adapters.kaiheila.message import MessageSegment
from nonebot.adapters.kaiheila.message import Audio as _Audio
from nonebot.adapters.kaiheila.message import Image as _Image
from nonebot.adapters.kaiheila.message import Quote as _Quote
from nonebot.adapters.kaiheila.message import Video as _Video
from nonebot.adapters.kaiheila.message import Mention as _Mention
from nonebot.adapters.kaiheila.message import KMarkdown as _KMarkdown
from nonebot.adapters.kaiheila.message import MentionAll as _MentionAll
from nonebot.adapters.kaiheila.message import MentionHere as _MentionHere
from nonebot.adapters.kaiheila.message import MentionRole as _MentionRole

from nonebot_plugin_alconna.argv import MessageArgv
from nonebot_plugin_alconna.typings import SegmentPattern

CHN = re.compile(r"\(chn\)(?P<id>.+?)\(chn\)")
EMJ = re.compile(r"\(emj\)(?P<name>.+?)\(emj\)\[(?P<id>[^\[]+?)\]")
EMJ1 = re.compile(r":(?P<id>.+?):")
NO_CHN = re.compile(r"\[\(chn\)(?P<id>.+?)\(chn\)\]\(.+\)")
NO_EMJ = re.compile(r"\[\(emj\)(?P<name>.+?)\(emj\)\[(?P<id>[^\[]+?)\]\]\(.+\)")
NO_EMJ1 = re.compile(r"\[:(?P<id>.+?):\]\(.+\)")


def builder(self: MessageArgv, data: Message):
    for index, unit in enumerate(data):
        if not self.is_text(unit):
            self.raw_data.append(unit)
            self.ndata += 1
            continue
        if unit.type == "text":
            text = unit.data["text"]
            if not text.strip():
                if not index or index == len(data) - 1:
                    continue
                if not self.is_text(data[index - 1]) or not self.is_text(data[index + 1]):
                    continue
            if self.raw_data and self.raw_data[-1].__class__ is str:
                self.raw_data[-1] = f"{self.raw_data[-1]}{text}"
            else:
                self.raw_data.append(text)
                self.ndata += 1
            continue
        content = unit.data["content"]
        mats = {}
        spans = {}
        for mat in CHN.finditer(content):
            mats[mat.start()] = MessageSegment("mention_channel", {"channel_id": mat.group("id")})
            spans[mat.start()] = mat.end()
        for mat in EMJ.finditer(content):
            mats[mat.start()] = MessageSegment("emoji", {"id": mat.group("id"), "name": mat.group("name")})
            spans[mat.start()] = mat.end()
        for mat in EMJ1.finditer(content):
            mats[mat.start()] = MessageSegment("emoji", {"id": mat.group("id")})
            spans[mat.start()] = mat.end()
        for mat in NO_CHN.finditer(content):
            if mat.start() + 1 in spans:
                del spans[mat.start() + 1]
                del mats[mat.start() + 1]
        for mat in NO_EMJ.finditer(content):
            if mat.start() + 1 in spans:
                del spans[mat.start() + 1]
                del mats[mat.start() + 1]
        for mat in NO_EMJ1.finditer(content):
            if mat.start() + 1 in spans:
                del spans[mat.start() + 1]
                del mats[mat.start() + 1]
        if not mats:
            self.raw_data.append(content)
            self.ndata += 1
            continue
        text = ""
        in_mat = False
        current = 0
        for _index, char in enumerate(content):
            if _index in spans:
                in_mat = True
                current = _index
                if text:
                    self.raw_data.append(text)
                    self.ndata += 1
                    text = ""
                self.raw_data.append(mats[_index])
                self.ndata += 1
                continue
            if _index >= spans[current]:
                in_mat = False
            if not in_mat:
                text += char
        if text:
            self.raw_data.append(text)
            self.ndata += 1


MessageArgv.custom_build(Message, builder=builder)  # type: ignore


def mention_channel(channel_id: str):
    return MessageSegment.KMarkdown(f"(chn){channel_id}(chn)", f"(chn){channel_id}(chn)")


def emoji(id: str, name: str = ""):
    if name:
        return MessageSegment.KMarkdown(f"(emj){name}(emj)[{id}]")
    else:
        return MessageSegment.KMarkdown(f":{id}:")


Text = str

Mention = At = SegmentPattern("mention", _Mention, MessageSegment.mention)
MentionRole = AtRole = SegmentPattern("mention_role", _MentionRole, MessageSegment.mention_role)
MentionChannel = AtChannel = SegmentPattern("mention_channel", MessageSegment, mention_channel)
MentionAll = AtAll = SegmentPattern("mention_all", _MentionAll, MessageSegment.mention_all)
MentionHere = AtHere = SegmentPattern("mention_here", _MentionHere, MessageSegment.mention_here)
Emoji = SegmentPattern("emoji", MessageSegment, emoji)

Image = SegmentPattern("image", _Image, MessageSegment.image)
Video = SegmentPattern("video", _Video, MessageSegment.video)
File = SegmentPattern("file", _File, MessageSegment.file)
Audio = SegmentPattern("audio", _Audio, MessageSegment.audio)
Card = SegmentPattern("card", _Card, MessageSegment.Card)
Quote = SegmentPattern("quote", _Quote, MessageSegment.quote)


class KMPattern(SegmentPattern):
    def match(self, input_) -> MessageSegment:
        if isinstance(input_, str):
            return MessageSegment.KMarkdown(input_, input_)
        return super().match(input_)


KMarkdown = KMPattern("kmarkdown", _KMarkdown, MessageSegment.KMarkdown)



from typing import Union

from nepattern.base import INTEGER
from nonebot.adapters.mirai2.message import MessageSegment
from nepattern import URL, MatchMode, BasePattern, UnionPattern

from nonebot_plugin_alconna.typings import SegmentPattern

Source = SegmentPattern("Source", MessageSegment, MessageSegment.source)
Quote = SegmentPattern("Quote", MessageSegment, MessageSegment.quote)
Plain = str
At = SegmentPattern("At", MessageSegment, MessageSegment.at)
AtAll = SegmentPattern("AtAll", MessageSegment, MessageSegment.at_all)
Face = SegmentPattern("Face", MessageSegment, MessageSegment.face)
Image = SegmentPattern("Image", MessageSegment, MessageSegment.image)
FlashImage = SegmentPattern("FlashImage", MessageSegment, MessageSegment.flash_image)
Voice = SegmentPattern("Voice", MessageSegment, MessageSegment.voice)
Xml = SegmentPattern("Xml", MessageSegment, MessageSegment.xml)
Json = SegmentPattern("Json", MessageSegment, MessageSegment.json)
App = SegmentPattern("App", MessageSegment, MessageSegment.app)
Dice = SegmentPattern("Dice", MessageSegment, MessageSegment.Dice)
Poke = SegmentPattern("Poke", MessageSegment, MessageSegment.poke)
MarketFace = SegmentPattern("MarketFace", MessageSegment, MessageSegment.market_face)
MusicShare = SegmentPattern("MusicShare", MessageSegment, MessageSegment.music_share)
Forward = SegmentPattern("Forward", MessageSegment, MessageSegment.forward)
File = SegmentPattern("File", MessageSegment, MessageSegment.file)
MiraiCode = SegmentPattern("MiraiCode", MessageSegment, MessageSegment.mirai_code)


ImgOrUrl = (
    UnionPattern[str](
        [
            BasePattern(
                mode=MatchMode.TYPE_CONVERT,
                origin=str,
                converter=lambda _, x: x.data["url"],
                alias="img",
                addition_accepts=Image,
            ),
            URL,
        ]
    )
    @ "img_url"
)
"""
内置类型, 允许传入图片元素(Image)或者链接(URL)，返回链接
"""

AtID = (
    UnionPattern[Union[str, MessageSegment]](
        [
            BasePattern(
                mode=MatchMode.TYPE_CONVERT,
                origin=int,
                alias="At",
                addition_accepts=At,
                converter=lambda _, x: int(x.data["target"]),
            ),
            BasePattern(
                r"@(\d+)",
                mode=MatchMode.REGEX_CONVERT,
                origin=int,
                alias="@xxx",
                accepts=str,
                converter=lambda _, x: int(x[1]),
            ),
            INTEGER,
        ]
    )
    @ "at_id"
)
"""
内置类型，允许传入提醒元素(At)或者'@xxxx'式样的字符串或者数字, 返回数字
"""



from dataclasses import field, dataclass
from typing_extensions import NotRequired
from typing import Set, Type, Union, Generic, Literal, TypeVar, Optional, TypedDict

from arclet.alconna import Empty, Alconna, Arparma
from arclet.alconna.duplication import Duplication

T = TypeVar("T")
T_Duplication = TypeVar("T_Duplication", bound=Duplication)


@dataclass
class Match(Generic[T]):
    """
    匹配项，表示参数是否存在于 `all_matched_args` 内
    result (T): 匹配结果
    available (bool): 匹配状态
    """

    result: T
    available: bool


class Query(Generic[T]):
    """
    查询项，表示参数是否可由 `Arparma.query` 查询并获得结果

    result (T): 查询结果

    available (bool): 查询状态

    path (str): 查询路径
    """

    result: T
    available: bool
    path: str

    def __init__(self, path: str, default: Union[T, Type[Empty]] = Empty):
        self.path = path
        self.result = default  # type: ignore
        self.available = False

    def __repr__(self):
        return f"Query({self.path}, {self.result})"


@dataclass(frozen=True)
class CommandResult:
    source: Alconna
    result: Arparma
    output: Optional[str] = field(default=None)

    @property
    def matched(self) -> bool:
        return self.result.matched


class CompConfig(TypedDict):
    tab: NotRequired[str]
    enter: NotRequired[str]
    exit: NotRequired[str]
    timeout: NotRequired[int]
    hide_tabs: NotRequired[bool]
    hides: NotRequired[Set[Literal["tab", "enter", "exit"]]]
    disables: NotRequired[Set[Literal["tab", "enter", "exit"]]]
    lite: NotRequired[bool]
